package com.lwby.aas.handle.impl;

import com.alicp.jetcache.anno.CacheRefresh;
import com.alicp.jetcache.anno.CacheType;
import com.alicp.jetcache.anno.Cached;
import com.lwby.aas.handle.AttributionHandle;
import com.lwby.aas.handle.AttributionType;
import com.lwby.aas.handle.BaseHelper;
import com.lwby.aas.handle.Flow;
import com.lwby.aas.po.AppChannel;
import com.lwby.aas.po.CrossCallback;
import com.lwby.aas.vo.Action;
import com.lwby.aas.vo.ClientInfo;
import com.lwby.aas.vo.DeliveryDeviceInfo;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.util.*;
import java.util.concurrent.ThreadLocalRandom;
import java.util.stream.Collectors;

@Component
public class CrossPlatformAttributionHandle extends BaseHelper implements AttributionHandle {
    @Resource
    JdbcTemplate lwbyJdbcTemplate;

    @Override
    public int order() {
        return 4;
    }

    @Override
    public Flow handle(Action action) {

        //获取跨平台帐号配置
        CrossPlatformAccount cpa = getCrossPlatformMedia().get(action.getMedia());
        //判断是否配置跨平台归因，如果没有则继续执行下一个计划归因处理器
        if(Objects.isNull(cpa) || !cpa.getAccount().contains(action.getAdvertiserId())){
            return Flow.NEXT;
        }

        //返回注册时间
        long daysSinceRegistration = 0;

        // 如果是当天注册新用户,则检查跨平台是否注册过，如果注册过，取最早注册时间做判断。
        if (isCurrentDayTime(action.getRegistrationDate())) {
            daysSinceRegistration = calculateDaysBetweenDates(getEarliestRegistrationTimeByDeviceId(action.getDeviceId()), new Date());
        }

        if(daysSinceRegistration <= cpa.spanCheckMaxDay){
            // 纯新用户逻辑
            //TODO;回传
            return Flow.END;
        } else if (daysSinceRegistration <= 150) {
            // 中间用户逻辑 (注册时间大于 cpa.spanCheckMaxDay 且小于 150 天)
            //TODO;直接扣量
        } else {
            // 老用户逻辑 (注册时间大于等于 150 天)
            AppChannel appChannel = getAppChannel(lwbyJdbcTemplate,action.getPlatformId(), action.getChannelId());

            //TODO；应该是等于空 或 = 100，如果空怎么办
            if (appChannel != null && appChannel.getSprDedu() == 100) {
                //TODO;回传
                return Flow.END;
            }

            //如果是20  就是回传 20
            Integer sprDedu = appChannel.getSprDedu();
            String dateStr = DATE_BOTTOM_FORMAT.format(new Date());

            //总数
            String channelTotal = String.format("%s_total_%d_%d_%d_%s",AttributionType.CROSS_PLATFORM, action.getPlatformId(), action.getChannelId(), sprDedu, dateStr);
            //回传
            String channelCallback = String.format("%s_callback_%d_%d_%d_%s", AttributionType.CROSS_PLATFORM, action.getPlatformId(), action.getChannelId(), sprDedu, dateStr);

            double percent = sprDedu.doubleValue();

            long channelTotalCount = incrby(channelTotal, 0, 60 * 60 * 24);
            long channelCallbackCount = incrby(channelCallback, 0, 60 * 60 * 24);

            //首次随机
            if(0 == channelTotalCount){
                incrby(channelTotal, 1);
                if (ThreadLocalRandom.current().nextInt(1, 3) == 1) {
                    //回传个数 ++
                    incrby(channelCallback, 1);
                    //TODO;回传事件
                    return Flow.END;
                }
            }else{
                //计算%
                BigDecimal divide = new BigDecimal(channelCallbackCount).divide(new BigDecimal(channelTotalCount == 0 ? 1 : channelTotalCount), 4, RoundingMode.HALF_UP);
                //总数 ++
                incrby(channelTotal, 1);
                // 扣量
                if (divide.compareTo(new BigDecimal(percent).setScale(4, RoundingMode.HALF_UP)) <= 0) {
                    //TODO;回传事件
                    incrby(channelCallback, 1);
                    return Flow.END;
                }
            }

            //TODO;扣量
            return Flow.END;
        }

        return Flow.END;
    }

    /**
     * 获取跨包配置
     * @return
     */
    @Cached(name="cross_platform_account", cacheType = CacheType.LOCAL)
    @CacheRefresh(refresh = 300)
    public Map<String,CrossPlatformAccount> getCrossPlatformMedia(){
        List<CrossCallback> ls = lwbyJdbcTemplate.queryForList("select id,span_check_max_day,new_account,old_account,media_name from cross_callback", CrossCallback.class);

        return ls.stream()
                .collect(Collectors.toMap(CrossCallback::getMediaName, CrossPlatformAccount::new));
    }

    /**
     * 获取设备ID的最早注册时间
     * @return
     */
    public Date getEarliestRegistrationTimeByDeviceId(String deviceId){
        Date date = lwbyJdbcTemplate.queryForObject(String.format("select min(registration_date) from user_profiles where device_id = '%s'",deviceId),Date.class);
        return Objects.isNull(date)?new Date():date;
    }

    @Data
    static class CrossPlatformAccount {
        /**
         * 媒体名称
         */
        private String mediaName;

        /**
         * 跨包纯新用户最大配置天数
         */
        private Integer spanCheckMaxDay;
        /**
         * 跨平台帐号
         */
        private Set<String> account;

        public CrossPlatformAccount(CrossCallback ccb){
            setMediaName(ccb.getMediaName());
            setSpanCheckMaxDay(ccb.getSpanCheckMaxDay());
            setAccount(Arrays.stream(ccb.getNewAccount().split(","))
                    .map(String::trim)
                    .filter(_s -> _s.matches("\\d+"))
                    .collect(Collectors.toSet()));
        }
    }

}